"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TestMessenger_1 = require("./TestMessenger");
const testApi2Impl = {
    addOne: (n) => __awaiter(void 0, void 0, void 0, function* () { return n + 1; }),
};
describe('RemoteMessenger', () => {
    it('should return API call results as promises', () => __awaiter(void 0, void 0, void 0, function* () {
        const messenger1 = new TestMessenger_1.default('test', testApi2Impl);
        const messenger2 = new TestMessenger_1.default('test', testApi2Impl);
        messenger1.connectTo(messenger2);
        expect(yield messenger1.remoteApi.addOne(3)).toBe(4);
        expect(yield messenger1.remoteApi.addOne(4)).toBe(5);
        expect(yield messenger2.remoteApi.addOne(4)).toBe(5);
    }));
    it('should support remote APIs with callbacks', () => __awaiter(void 0, void 0, void 0, function* () {
        const callbacks = [];
        const testApi1 = {
            foo: {
                bar: jest.fn(),
                multiplyAndCallCallback: jest.fn((n, callback) => __awaiter(void 0, void 0, void 0, function* () {
                    callback(n * 2);
                    callback(n * 3);
                })),
            },
            registerCallback: (options) => __awaiter(void 0, void 0, void 0, function* () {
                callbacks.push(options.callback);
                return true;
            }),
        };
        const api2Messenger = new TestMessenger_1.default('test', testApi1);
        const api1Messenger = new TestMessenger_1.default('test', testApi2Impl);
        api2Messenger.connectTo(api1Messenger);
        // Simple callback test
        expect(testApi1.foo.multiplyAndCallCallback).toHaveBeenCalledTimes(0);
        const multiplyCallback = jest.fn();
        yield api1Messenger.remoteApi.foo.multiplyAndCallCallback(1, multiplyCallback);
        expect(multiplyCallback).toHaveBeenCalledTimes(2);
        expect(multiplyCallback).toHaveBeenCalledWith(2);
        expect(multiplyCallback).toHaveBeenCalledWith(3);
        // Test accessing the registerCallback method
        const registerCallbackTest = jest.fn();
        yield api1Messenger.remoteApi.registerCallback({ callback: registerCallbackTest });
        expect(callbacks).toHaveLength(1);
    }));
    it('should support returning an object with callbacks', () => __awaiter(void 0, void 0, void 0, function* () {
        const testApi = {
            getApi: () => __awaiter(void 0, void 0, void 0, function* () {
                return {
                    multiply: (a, b) => __awaiter(void 0, void 0, void 0, function* () { return a * b; }),
                    add: (a, b) => __awaiter(void 0, void 0, void 0, function* () { return a + b; }),
                };
            }),
        };
        const messenger1 = new TestMessenger_1.default('test', testApi);
        const messenger2 = new TestMessenger_1.default('test', testApi);
        messenger1.connectTo(messenger2);
        const api = yield messenger1.remoteApi.getApi();
        expect(yield api.multiply(2, 3)).toBe(6);
        expect(yield api.add(12, 3)).toBe(15);
    }));
    it('should preserve structure of transferred objects', () => __awaiter(void 0, void 0, void 0, function* () {
        const transferObjectApi = {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            transfer: (o) => __awaiter(void 0, void 0, void 0, function* () { return o; }),
        };
        const messenger1 = new TestMessenger_1.default('test', transferObjectApi);
        const messenger2 = new TestMessenger_1.default('test', transferObjectApi);
        messenger1.connectTo(messenger2);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const testObjects = [
            { foo: { bar: undefined, baz: null } },
            { foo: { bar: [1, 2, 3], baz: 'test' } },
            { _a: 4.5, __b: '', __callbacks: 'foo', __proto__: { a: 6 } },
        ];
        for (const testObject of testObjects) {
            expect(yield messenger1.remoteApi.transfer(testObject)).toMatchObject(testObject);
        }
    }));
    it('should preserve the value of `this`', () => __awaiter(void 0, void 0, void 0, function* () {
        // We construct an API that intentionally relies on `this`
        const testApi = {
            add(arg1, arg2) {
                return __awaiter(this, void 0, void 0, function* () {
                    return this._printSum(arg1 + arg2);
                });
            },
            _printSum(sum) {
                return `sum: ${sum}`;
            },
            subObject: {
                multiplyRounded(arg1, arg2) {
                    return __awaiter(this, void 0, void 0, function* () {
                        return this._round(arg1) * this._round(arg2);
                    });
                },
                _round(x) {
                    return Math.round(x);
                },
            },
        };
        const messenger1 = new TestMessenger_1.default('test', testApi);
        const messenger2 = new TestMessenger_1.default('test', testApi);
        messenger1.connectTo(messenger2);
        const remoteApi = messenger1.remoteApi;
        // Should preserve this by default
        expect(yield remoteApi.add(1, 2)).toBe('sum: 3');
        // .call and .apply should still call the function
        expect(yield remoteApi.add.apply(remoteApi, [3, 2])).toBe('sum: 5');
        expect(yield remoteApi.add.call(remoteApi, 3, 2)).toBe('sum: 5');
        // The same should be true for sub-objects
        expect(yield remoteApi.subObject.multiplyRounded(1.1, 2)).toBe(2);
        expect(yield remoteApi.subObject.multiplyRounded.call(remoteApi.subObject, 3.1, 4.2)).toBe(12);
    }));
    it('should delete callbacks when dropped remotely', () => __awaiter(void 0, void 0, void 0, function* () {
        const testApi = {
            test: jest.fn(),
        };
        const messenger1 = new TestMessenger_1.default('testid', testApi);
        const messenger2 = new TestMessenger_1.default('testid', testApi);
        messenger1.connectTo(messenger2);
        const callback = () => __awaiter(void 0, void 0, void 0, function* () { });
        messenger1.remoteApi.test(callback);
        // Callbacks should be stored with the source messenger
        const callbackId = messenger1.getIdForCallback_(callback);
        expect(callbackId).toBeTruthy();
        expect(messenger2.getIdForCallback_(callback)).toBe(undefined);
        // Dropping a callback at the remote messenger should clear the
        // callback on the original messenger
        messenger2.mockCallbackDropped(callbackId);
        // To avoid random test failure, wait for a round-tip before checking
        // whether the callback is still registered.
        yield messenger1.remoteApi.test(() => __awaiter(void 0, void 0, void 0, function* () { }));
        expect(messenger1.getIdForCallback_(callback)).toBe(undefined);
    }));
});
//# sourceMappingURL=RemoteMessenger.test.js.map